import nuke
from PySide2 import QtWidgets, QtCore


class GuiExpressionSearch(QtWidgets.QWidget):
    def __init__(self):
        super(GuiExpressionSearch, self).__init__()

        self.node_class = QtWidgets.QComboBox()
        self.node_class.addItems(['All Nodes', 'ScanlineRender', 'VectorBlur', 'Median'])

        self.search_button = QtWidgets.QPushButton('Search')
        self.search_button.clicked.connect(self.search_expressions)
        self.result_text = QtWidgets.QLabel('')

        self.node_list = QtWidgets.QTableWidget()
        self.node_list.setColumnCount(3)
        self.node_list.setHorizontalHeaderItem(0, QtWidgets.QTableWidgetItem('Name'))
        self.node_list.setHorizontalHeaderItem(1, QtWidgets.QTableWidgetItem('Knob'))
        self.node_list.setHorizontalHeaderItem(2, QtWidgets.QTableWidgetItem('Expression'))
        self.node_list.horizontalHeader().setSectionResizeMode(0, QtWidgets.QHeaderView.Stretch)
        self.node_list.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.node_list.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.node_list.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.node_list.doubleClicked.connect(self.jump_to_node)

        info = '   v1.1 / www.richter-steffen.com / Double click jumps to node in nodegraph'
        self.info = QtWidgets.QLabel(info)

        top_layout = QtWidgets.QHBoxLayout()
        top_layout.addWidget(self.node_class)
        top_layout.addWidget(self.search_button)

        main_layout = QtWidgets.QVBoxLayout()
        main_layout.addLayout(top_layout)
        main_layout.addWidget(self.result_text)
        main_layout.addWidget(self.node_list)
        main_layout.addWidget(self.info)

        self.setLayout(main_layout)

    def search_expressions(self):
        """searches the node tree for nodes with $gui expression"""
        self.node_list.clearContents()
        self.node_list.setRowCount(0)
        table_data = []
        choice = self.node_class.currentText()
        if choice == 'All Nodes':
            choice_nodes = nuke.allNodes()
        elif choice == 'VectorBlur':
            choice_nodes = nuke.allNodes('VectorBlur2')
        elif choice == 'ScanlineRender':
            choice_nodes = nuke.allNodes('ScanlineRender')
        elif choice == 'Median':
            choice_nodes = nuke.allNodes('Median')
        try:
            for n in choice_nodes:
                for k in n.knobs().keys():
                    if n[k].hasExpression():
                        if '$gui' in n[k].animation(0).expression():
                            items = [n.name(), k, n[k].animation(0).expression()]
                            table_data.append(items)
        except AttributeError:
            pass

        if len(table_data) == 0:
            self.result_text.setText('No $gui Expressions found')
        else:
            self.result_text.setText('')
            self.node_list.setRowCount(len(table_data))
            for row_number, row_data in enumerate(table_data):
                for column_number, data in enumerate(row_data):
                    self.node_list.setItem(row_number, column_number, QtWidgets.QTableWidgetItem(data))

    def jump_to_node(self):
        current_row = self.node_list.currentRow()
        table_selection = self.node_list.item(current_row,0).text()
        node = nuke.toNode(table_selection)
        xC = node.xpos() + node.screenWidth()/2
        yC = node.ypos() + node.screenHeight()/2
        nuke.zoom(1, [xC, yC])


def launch_ui():
    global window
    window = GuiExpressionSearch()
    window.setWindowTitle('Find Gui Expressions')
    window.resize(600, 400)
    window.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
    window.show()


if __name__ == '__main__':
    launch_ui()